# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qm/pb/compiler.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from qm.pb import inc_qua_pb2 as qm_dot_pb_dot_inc__qua__pb2
from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2
from qm.pb import qm_manager_pb2 as qm_dot_pb_dot_qm__manager__pb2
from qm.pb import general_messages_pb2 as qm_dot_pb_dot_general__messages__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='qm/pb/compiler.proto',
  package='qm.grpc.compiler',
  syntax='proto3',
  serialized_options=b'\n\020qm.grpc.compilerB\rCompilerProtoP\001\242\002\003HLW',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x14qm/pb/compiler.proto\x12\x10qm.grpc.compiler\x1a\x13qm/pb/inc_qua.proto\x1a\x1cgoogle/protobuf/struct.proto\x1a\x16qm/pb/qm_manager.proto\x1a\x1cqm/pb/general_messages.proto\"K\n\tQuaValues\x12\x11\n\tint_value\x18\x01 \x01(\r\x12\x14\n\x0c\x64ouble_value\x18\x02 \x01(\x01\x12\x15\n\rboolean_value\x18\x03 \x01(\x08\"I\n\x0e\x43ompileRequest\x12(\n\x07program\x18\x01 \x01(\x0b\x32\x17.qm.grpc.qua.QuaProgram\x12\r\n\x05jobId\x18\x02 \x01(\t\"\xdf\x01\n\x0f\x43ompileResponse\x12\n\n\x02ok\x18\x01 \x01(\x08\x12\x33\n\x08messages\x18\x02 \x03(\x0b\x32!.qm.grpc.compiler.CompilerMessage\x12\x0e\n\x06\x62inary\x18\x03 \x01(\x0c\x12\x10\n\x08metadata\x18\x04 \x01(\t\x12;\n\x05\x64\x65\x62ug\x18\x1e \x03(\x0b\x32,.qm.grpc.compiler.CompileResponse.DebugEntry\x1a,\n\nDebugEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"Y\n\x0f\x43ompilerMessage\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x35\n\x05level\x18\x02 \x01(\x0e\x32&.qm.grpc.general_messages.MessageLevel\"S\n\x12ValidationResponse\x12=\n\x08messages\x18\x01 \x03(\x0b\x32+.qm.grpc.qm_manager.ConfigValidationMessage\"G\n\rDynamicConfig\x12\x0f\n\x07version\x18\x01 \x01(\r\x12%\n\x04root\x18\x02 \x01(\x0b\x32\x17.google.protobuf.StructB)\n\x10qm.grpc.compilerB\rCompilerProtoP\x01\xa2\x02\x03HLWb\x06proto3'
  ,
  dependencies=[qm_dot_pb_dot_inc__qua__pb2.DESCRIPTOR,google_dot_protobuf_dot_struct__pb2.DESCRIPTOR,qm_dot_pb_dot_qm__manager__pb2.DESCRIPTOR,qm_dot_pb_dot_general__messages__pb2.DESCRIPTOR,])




_QUAVALUES = _descriptor.Descriptor(
  name='QuaValues',
  full_name='qm.grpc.compiler.QuaValues',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='int_value', full_name='qm.grpc.compiler.QuaValues.int_value', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='double_value', full_name='qm.grpc.compiler.QuaValues.double_value', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='boolean_value', full_name='qm.grpc.compiler.QuaValues.boolean_value', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=147,
  serialized_end=222,
)


_COMPILEREQUEST = _descriptor.Descriptor(
  name='CompileRequest',
  full_name='qm.grpc.compiler.CompileRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='program', full_name='qm.grpc.compiler.CompileRequest.program', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='jobId', full_name='qm.grpc.compiler.CompileRequest.jobId', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=224,
  serialized_end=297,
)


_COMPILERESPONSE_DEBUGENTRY = _descriptor.Descriptor(
  name='DebugEntry',
  full_name='qm.grpc.compiler.CompileResponse.DebugEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='qm.grpc.compiler.CompileResponse.DebugEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='qm.grpc.compiler.CompileResponse.DebugEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=479,
  serialized_end=523,
)

_COMPILERESPONSE = _descriptor.Descriptor(
  name='CompileResponse',
  full_name='qm.grpc.compiler.CompileResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='ok', full_name='qm.grpc.compiler.CompileResponse.ok', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='messages', full_name='qm.grpc.compiler.CompileResponse.messages', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='binary', full_name='qm.grpc.compiler.CompileResponse.binary', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='metadata', full_name='qm.grpc.compiler.CompileResponse.metadata', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='debug', full_name='qm.grpc.compiler.CompileResponse.debug', index=4,
      number=30, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_COMPILERESPONSE_DEBUGENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=300,
  serialized_end=523,
)


_COMPILERMESSAGE = _descriptor.Descriptor(
  name='CompilerMessage',
  full_name='qm.grpc.compiler.CompilerMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='message', full_name='qm.grpc.compiler.CompilerMessage.message', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='level', full_name='qm.grpc.compiler.CompilerMessage.level', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=525,
  serialized_end=614,
)


_VALIDATIONRESPONSE = _descriptor.Descriptor(
  name='ValidationResponse',
  full_name='qm.grpc.compiler.ValidationResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='messages', full_name='qm.grpc.compiler.ValidationResponse.messages', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=616,
  serialized_end=699,
)


_DYNAMICCONFIG = _descriptor.Descriptor(
  name='DynamicConfig',
  full_name='qm.grpc.compiler.DynamicConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='version', full_name='qm.grpc.compiler.DynamicConfig.version', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='root', full_name='qm.grpc.compiler.DynamicConfig.root', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=701,
  serialized_end=772,
)

_COMPILEREQUEST.fields_by_name['program'].message_type = qm_dot_pb_dot_inc__qua__pb2._QUAPROGRAM
_COMPILERESPONSE_DEBUGENTRY.containing_type = _COMPILERESPONSE
_COMPILERESPONSE.fields_by_name['messages'].message_type = _COMPILERMESSAGE
_COMPILERESPONSE.fields_by_name['debug'].message_type = _COMPILERESPONSE_DEBUGENTRY
_COMPILERMESSAGE.fields_by_name['level'].enum_type = qm_dot_pb_dot_general__messages__pb2._MESSAGELEVEL
_VALIDATIONRESPONSE.fields_by_name['messages'].message_type = qm_dot_pb_dot_qm__manager__pb2._CONFIGVALIDATIONMESSAGE
_DYNAMICCONFIG.fields_by_name['root'].message_type = google_dot_protobuf_dot_struct__pb2._STRUCT
DESCRIPTOR.message_types_by_name['QuaValues'] = _QUAVALUES
DESCRIPTOR.message_types_by_name['CompileRequest'] = _COMPILEREQUEST
DESCRIPTOR.message_types_by_name['CompileResponse'] = _COMPILERESPONSE
DESCRIPTOR.message_types_by_name['CompilerMessage'] = _COMPILERMESSAGE
DESCRIPTOR.message_types_by_name['ValidationResponse'] = _VALIDATIONRESPONSE
DESCRIPTOR.message_types_by_name['DynamicConfig'] = _DYNAMICCONFIG
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

QuaValues = _reflection.GeneratedProtocolMessageType('QuaValues', (_message.Message,), {
  'DESCRIPTOR' : _QUAVALUES,
  '__module__' : 'qm.pb.compiler_pb2'
  # @@protoc_insertion_point(class_scope:qm.grpc.compiler.QuaValues)
  })
_sym_db.RegisterMessage(QuaValues)

CompileRequest = _reflection.GeneratedProtocolMessageType('CompileRequest', (_message.Message,), {
  'DESCRIPTOR' : _COMPILEREQUEST,
  '__module__' : 'qm.pb.compiler_pb2'
  # @@protoc_insertion_point(class_scope:qm.grpc.compiler.CompileRequest)
  })
_sym_db.RegisterMessage(CompileRequest)

CompileResponse = _reflection.GeneratedProtocolMessageType('CompileResponse', (_message.Message,), {

  'DebugEntry' : _reflection.GeneratedProtocolMessageType('DebugEntry', (_message.Message,), {
    'DESCRIPTOR' : _COMPILERESPONSE_DEBUGENTRY,
    '__module__' : 'qm.pb.compiler_pb2'
    # @@protoc_insertion_point(class_scope:qm.grpc.compiler.CompileResponse.DebugEntry)
    })
  ,
  'DESCRIPTOR' : _COMPILERESPONSE,
  '__module__' : 'qm.pb.compiler_pb2'
  # @@protoc_insertion_point(class_scope:qm.grpc.compiler.CompileResponse)
  })
_sym_db.RegisterMessage(CompileResponse)
_sym_db.RegisterMessage(CompileResponse.DebugEntry)

CompilerMessage = _reflection.GeneratedProtocolMessageType('CompilerMessage', (_message.Message,), {
  'DESCRIPTOR' : _COMPILERMESSAGE,
  '__module__' : 'qm.pb.compiler_pb2'
  # @@protoc_insertion_point(class_scope:qm.grpc.compiler.CompilerMessage)
  })
_sym_db.RegisterMessage(CompilerMessage)

ValidationResponse = _reflection.GeneratedProtocolMessageType('ValidationResponse', (_message.Message,), {
  'DESCRIPTOR' : _VALIDATIONRESPONSE,
  '__module__' : 'qm.pb.compiler_pb2'
  # @@protoc_insertion_point(class_scope:qm.grpc.compiler.ValidationResponse)
  })
_sym_db.RegisterMessage(ValidationResponse)

DynamicConfig = _reflection.GeneratedProtocolMessageType('DynamicConfig', (_message.Message,), {
  'DESCRIPTOR' : _DYNAMICCONFIG,
  '__module__' : 'qm.pb.compiler_pb2'
  # @@protoc_insertion_point(class_scope:qm.grpc.compiler.DynamicConfig)
  })
_sym_db.RegisterMessage(DynamicConfig)


DESCRIPTOR._options = None
_COMPILERESPONSE_DEBUGENTRY._options = None
# @@protoc_insertion_point(module_scope)
